function varargout = ps_LST_lesfill(job,arg)

%     ps_LST_lesfill.m - Lesion filling, part of the LST toolbox
%     Copyright (C) 2012  Paul Schmidt
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

nimgs = numel(job.data_T1);
if nimgs ~= numel(job.data_lm) 
    error('Input data must have the same length.')
end

fprintf('Fill lesions for...\n')

for ii = 1:nimgs

    [job_pth, job_T1,ext_T1] = spm_fileparts(job.data_T1{ii});
    [job_pth, job_lm] = fileparts(job.data_lm{ii});
    nam_T1 = strtok(job_T1, '.');
    fprintf(job_pth, '\n')
    
    % load images
    Les = spm_read_vols(spm_vol(fullfile(job_pth, [job_lm, '.nii'])));
    PVE = spm_read_vols(spm_vol(fullfile(job_pth, ['p0', nam_T1, '.nii'])));
    mT1 = spm_read_vols(spm_vol(fullfile(job_pth, ['m', nam_T1, '.nii'])));
    kappa_tmp = str2num(strtok(strtok(job_lm, 'lesion_'), '.'))/1000;
    
    y = mT1(PVE > 2.75);
    fprintf('\nEstimate distribution of healthy white matter... ')
    [mu, sigma2, lambda] = gmm_em(y, 2);
    fprintf(' done.\n')
    
    indx = find(Les > 0);
    clear [Les PVE];
    n = numel(indx);
    u = rand(n, 1);    
    y_sim = (u <= lambda(1)) .* rnorm(n, mu(1), sqrt(sigma2(1))) + (u > lambda(1)) .* rnorm(n, mu(2), sqrt(sigma2(2)));
    mT1_filled = mT1;
    mT1_filled(indx) = y_sim;
    neighborhood = getNeighborhood2(mT1_filled, indx, 3);
    F = repmat([1/4; 3/128; 0.0625; 3/128; 0.0625; 0.0625; 3/128; 3/128; 0.0625; ...
    3/256; 3/128; 3/256; 3/128; 3/128; 3/256; 3/256; 3/128; 0.0625; ...
    3/256; 3/128; 3/256; 3/128; 3/128; 3/256; 3/256; 3/128; 0.0625], 1, n);
    mT1_filled(indx) = sum([y_sim'; neighborhood] .* F);
    
    %T1 = spm_read_vols(spm_vol(fullfile(job.data_T1{ii})));
    T1 = spm_read_vols(spm_vol(job.data_T1{ii}));
    BF = T1 ./ mT1;
    clear [T1 mT1];
    T1_filled = mT1_filled .* BF;
    clear BF;
    T1_filled(isnan(T1_filled)) = 0;
    
    if job.output_lesfill.T1_filled.native_filled
        % filled T1 image in native space
        fprintf('Write filled T1 image in native space... ')
        VT1_filled = spm_vol(fullfile(job_pth, ['m', nam_T1, '.nii']));
        if kappa_tmp < 1
            if kappa_tmp < .1
                ext = ['00', num2str(kappa_tmp), '.nii'];
            else
                ext = ['0', num2str(kappa_tmp*1000), '.nii'];
            end
        else
            ext = [num2str(kappa_tmp*1000), ext_T1];
        end
        VT1_filled(1).fname = fullfile(job_pth, [nam_T1, '_filled_', '.nii']);
        spm_write_vol(VT1_filled(1), T1_filled);
            
        if job.output_lesfill.T1_filled.normalized_filled
            % normalize filled T1 image
            fprintf(' and the normalized version in MNI space ... ')
            field1 = {fullfile(job_pth, ['y_', nam_T1, '.nii'])};
            images = {VT1_filled(1).fname};
            jobDeform = struct('interp', 5, ...
               'modulate', 0);
            jobDeform.field1 = field1;
            jobDeform.images = images;
            cg_vbm8_defs(jobDeform)
        end
        fprintf('done!\n')
    end
    clear T1_filled;
    
    if job.output_lesfill.T1_filled.bias_filled
        % write bias-corrected filled T1 image in native space
        fprintf('Write bias-corrected filled T1 image in native space... ')
        VT1_filled = spm_vol(fullfile(job_pth, ['m', nam_T1, '.nii']));
        if kappa_tmp < 1
            if kappa_tmp < .1
                ext = ['00', num2str(kappa_tmp), '.nii'];
            else
                ext = ['0', num2str(kappa_tmp*1000), '.nii'];
            end
        else
            ext = [num2str(kappa_tmp*1000), '.nii'];
        end
        VT1_filled(1).fname = fullfile(job_pth, ['m', nam_T1, '_filled_', '.nii']);
        spm_write_vol(VT1_filled(1), mT1_filled);
            
        if job.output_lesfill.T1_filled.normalized_filled
            % normalize filled T1 image
            fprintf(' and the normalized version in MNI space ... ')
            field1 = {fullfile(job_pth, ['y_', nam_T1, '.nii'])};
            images = {VT1_filled(1).fname};
            jobDeform = struct('interp', 5, ...
               'modulate', 0);
            jobDeform.field1 = field1;
            jobDeform.images = images;
            cg_vbm8_defs(jobDeform)
        end
        fprintf('done!\n')
    end

end

c = clock();
disp(['Finished successfully on ', datestr(c)]);
varargout{:} = ['Finished successfully on ', datestr(c)];

return

