function lesiongrow = ps_LST_lesiongrowing

%     ps_LST_lesiongrowing.m - part of the LST toolbox
%     Copyright (C) 2012  Paul Schmidt
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.

data_T1 = cfg_files;
data_T1.tag  = 'data_T1';
data_T1.name = 'T1 volumes';
data_T1.help = {[...
'Select raw T1 images. You can select images of more than one subject. The FLAIR images must be selected in the same order.']};
data_T1.filter = 'image';
data_T1.ufilter = '.*';
data_T1.num     = [1 Inf];

%------------------------------------------------------------------------

data_FLAIR = cfg_files;
data_FLAIR.tag  = 'data_FLAIR';
data_FLAIR.name = 'FLAIR volumes';
data_FLAIR.help = {[...
'Select raw FLAIR images. These images must be selected in the same order as the T1 images.']};
data_FLAIR.filter = 'image';
data_FLAIR.ufilter = '.*';
data_FLAIR.num     = [1 Inf];

% ------------------------------------------------------------------------

initial = cfg_entry;
initial.tag  = 'initial';
initial.name = 'Initial threshold';
initial.strtype = 'e';
initial.num = [1 20];
initial.def  = @(val) [0.3, zeros(1, 19)];
initial.help = {[...
'Threshold (kappa) for the initial lesion mask. You can choose up to twenty'...
' different values.'...
''...
' See the Schmidt et al. (2012) for details and the manual on how to ' ...
'determine the optimal value for this threshold.']};

belief = cfg_menu;
belief.tag = 'belief';
belief.name = 'Lesion belief map';
belief.labels = {'GM','WM', 'CSF', 'GM+WM', 'GM+CSF', 'WM+CSF', 'All'};
belief.values = {0 1 2 3 4 5 6};
belief.def = @(val) 0;
belief.help    = {['The lesion belief map is thresholded by the initial ' ...
    'threshold in order to produce the initial lesion mask. The original ' ...
    'implementation uses the belief map of GM only, see Schmidt et al. (2012) ' ...
    'for details. Here, the user can choose different combinations of all '... 
    'lesion belief maps. However, those should be used with care, as the ' ...
    'resulting segmentations may differ a lot from the one obtained by the default option.']};

mrf = cfg_entry;
mrf.tag  = 'mrf';
mrf.name = 'MRF parameter';
mrf.strtype = 'e';
mrf.num = [1 1];
mrf.def  = @(x) 1;
mrf.help = {[...
'Parameter for the Markov Random Field.']};

maxiter = cfg_entry;
maxiter.tag  = 'maxiter';
maxiter.name = 'Maximum iterations';
maxiter.strtype = 'e';
maxiter.num = [1 1];
maxiter.def  = @(val) 50; 
maxiter.help = {[...
'Number of maximum iterations.']};

threshold      = cfg_entry;
threshold.tag = 'threshold';
threshold.name = 'Threshold for binary lesion map';
threshold.strtype = 'e';
threshold.num = [1 1];
threshold.def  = @(val) 0; 
threshold.help = {[...
'The threshold for computing binary lesion maps. This value should be '...
'a number in the interval (0,1].']};

segopts = cfg_branch;
segopts.tag = 'segopts';
segopts.name = 'Options for lesion segmentation';
segopts.val = {initial,belief,mrf,maxiter,threshold};
segopts.help = {['Options for the lesion growth model.']};

% ------------------------------------------------------------------------

prob    = cfg_menu;
prob.tag = 'prob';
prob.name = 'Lesion probability map';
prob.labels = {'none','yes'};
prob.values = {0 1};
prob.def = @(val) 1;
prob.help    = {['The probability lesion map assigns a value in the range '...
    'of [0,1] to all voxels.']};

binary    = cfg_menu;
binary.tag = 'binary';
binary.name = 'Binary lesion map';
binary.labels = {'none','yes'};
binary.values = {0 1};
binary.def = @(val) 0;
binary.help    = {['The binary lesion map consists of voxels with either '...
    'a 0 or a 1, where the lesion probability for a voxel with a 1 is at'...
    'least as high as the threshold for the binary lesion map.']};

normalized    = cfg_menu;
normalized.tag = 'normalized';
normalized.name = 'Normalized lesion map';
normalized.labels = {'none','yes'};
normalized.values = {0 1};
normalized.def = @(val) 0; 
normalized.help    = {['Choose wheather the lesion probaility map and/or '...
    'the binary lesion map should be normalized. The forward deformation '...
    'field that results from the PVE-label estimation is used for normalization.']};

lesions      = cfg_branch;
lesions.tag = 'lesions';
lesions.name = 'Lesion Map';
lesions.val = {prob,binary,normalized};
lesions.help     = {'Writing options for the segmented lesions.'};

other      = cfg_menu;
other.tag = 'other';
other.name = 'Other images';
other.labels = {'no','yes'};
other.values = {0 1};
other.def  = @(val) 1;
other.help = {['Per default, images that are needed for lesion segmentation are '...
    'saved. To be precise, these images are the T1 and FLAIR images '...
    'corrected for bias, the coregistered bias-corrected FLAIR image, '...
    'the estimated PVE label as well as the forward and inverse '...
    'deformation field. These images are needed to run the lesion growth '...
    'algorithm again (e.g. with a different initial threshold) without '...
    'estimating the PVE label and the bias correction of the FLAIR image.']};

output      = cfg_branch;
output.tag = 'output';
output.name = 'Writing options';
output.val = {lesions,other};
output.help = {['']};

%------------------------------------------------------------------------

lesiongrow = cfg_exbranch;
lesiongrow.name = 'PVE label estimation and lesion segmentation';
lesiongrow.tag  = 'lesiongrow';
lesiongrow.val  = {data_T1,data_FLAIR,segopts,output};
lesiongrow.prog = @ps_LST_lesiongrowing_run;
lesiongrow.vout = @vout_lesiongrow;
lesiongrow.help = {['This tool implements a lesion growth algorithm for '...
    'the segmentation of lesions in Multiple Sclerosis (MS).'...
    ''...
    'THe algorithm is explained in detail in Schmidt et al. (2011). See '...
    'the manual for the usage of this tool.']};

return;
%------------------------------------------------------------------------
 
%------------------------------------------------------------------------
function dep = vout_lesiongrow(job)

    opts  = job.output;
    cdep = cfg_dep;

    if opts.lesions.prob
        cdep(end).sname      = 'Lesion probability map';
        cdep(end).src_output = substruct('.','lpm','()',{':'});
        cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});

        if opts.lesions.normalized
            cdep(end+1)          = cfg_dep; 
            cdep(end).sname      = 'Normalized lesion probability map';
            cdep(end).src_output = substruct('.','wlpm','()',{':'});
            cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
        end
    end

    if opts.lesions.binary
        if opts.lesions.prob
            cdep(end+1) = cfg_dep; 
        end
        cdep(end).sname      = 'Binary lesion map';
        cdep(end).src_output = substruct('.','blm','()',{':'});
        cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});

        if opts.lesions.normalized
            cdep(end+1)          = cfg_dep; 
            cdep(end).sname      = 'Normalized binary lesion map';
            cdep(end).src_output = substruct('.','wblm','()',{':'});
            cdep(end).tgt_spec   = cfg_findspec({{'filter','image','strtype','e'}});
        end
    end
dep = cdep;
%------------------------------------------------------------------------
